/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017-2022 Yegor Bugayenko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.cactoos.iterator;

import java.util.Iterator;
import java.util.NoSuchElementException;
import org.cactoos.Scalar;
import org.cactoos.scalar.Unchecked;

/**
 * Repeat an element.
 *
 * <p>If you need to repeat endlessly, use {@link Endless}.</p>
 *
 * @param <T> Element type
 * @since 0.4
 */
public final class Repeated<T> implements Iterator<T> {

    /**
     * The element to repeat.
     */
    private final Unchecked<? extends T> elm;

    /**
     * How many more repeats will happen.
     */
    private int repeat;

    /**
     * Ctor.
     * @param max How many times to repeat
     * @param element Element to repeat
     */
    public Repeated(final int max, final T element) {
        this(max, () -> element);
    }

    /**
     * Ctor.
     * @param max How many times to repeat
     * @param scalar Scalar to repeat
     */
    public Repeated(final int max, final Scalar<? extends T> scalar) {
        this.repeat = max;
        this.elm = new Unchecked<>(scalar);
    }

    @Override
    public boolean hasNext() {
        return this.repeat > 0;
    }

    @Override
    public T next() {
        if (!this.hasNext()) {
            throw new NoSuchElementException(
                "The iterator can't repeat anymore."
            );
        }
        --this.repeat;
        return this.elm.value();
    }
}
